// file: patterns.cc

#include <fstream.h>
#include "general.h"
#include "patterns.h"

int     npatterns;
int     maxpatterns = MAXPATTERNS;
float  **ipattern;
float  **tpattern;
char   **pname;
int    *used;
char   cpname[BUFSIZ];

extern int ninputs; 
extern int noutputs;

boolean startt(char *s1, char *s2)   
// faengt s1 genauso an wie s2?
{
  while (*s1 && *s2) {
    if (*s1++ != *s2++)
      return(FALSE);
  }
  if(*s1 && !*s2)  // wenn s1 laenger ist als s2 soll die Fkt. FALSE liefern
    return(FALSE);
  return(TRUE);
}
void init_pattern_pairs() {
  #ifdef DEBUG
  cout << "\npatterns|init_pattern_pairs()";
  #endif
  init_pats(PAIRS);
}

void init_patterns() {
  #ifdef DEBUG
  cout << "\npatterns|init_patterns()";
  #endif
  init_pats(!PAIRS);
}

void init_pats(int pairs) 
{
  ipattern=NULL;
  tpattern=NULL;
  #ifdef DEBUG
  cout << "\npatterns|init_pats()";
  #endif
  if (pairs)
    get_pattern_pairs();
  else
    get_patterns();
}

void reset_patterns(int pairs) 
{
    int i;
    
  #ifdef DEBUG
  cout << "\npatterns|reset_patterns()";
  #endif
  if (pname) {
    for (i = 0; i < npatterns; i++) {
      if(pname[i] != NULL)
        delete[] pname[i];
    }
    delete[] pname;
  }
  pname = new char *[maxpatterns];
  if (ipattern) {
    for(i = 0; i < npatterns; i++) {
      if(ipattern[i] != NULL)
        delete[] ipattern[i];
    }
    delete[] ipattern;
  }
  ipattern = new float *[maxpatterns];
  if (pairs) {
    if (tpattern) {
      for (i = 0; i < npatterns; i++) {
       if(tpattern[i] != NULL)
        delete[] tpattern[i];
      }
      delete[] tpattern;
    }
    tpattern = new float *[maxpatterns];
  }
  if (used) delete[] used;
  used = new int[maxpatterns];
}

int get_patterns() {
  #ifdef DEBUG
  cout << "\npatterns|get_patterns()";
  #endif
  return(get_pats(!PAIRS));
}

int get_pattern_pairs() {
  #ifdef DEBUG
  cout << "\npatterns|get_pattern_pairs()";
  #endif
  return(get_pats(PAIRS));
}

int get_pats(int pairs)
{
  int     i,j,rval;
  char    temp[LINE_SIZE];
  ifstream muster_file;
  char *fname = new char [BUFSIZ];

  #ifdef DEBUG
  cout << "\npatterns|get_pats()";
  #endif

  if (!System_Defined)
    return(put_error("Netzwerk nicht definiert!"));

  if (lflag) strcpy(fname, "train.mst");
  else strcpy(fname, "test.mst");
  
  if(fname == NULL) return(BREAK);

  muster_file.open(fname);
  if (muster_file.fail()) {
      return(put_error("Kann die Datei mit Mustern nicht oeffnen."));
  }
  reset_patterns(pairs);

  for (i = 0; !muster_file.eof(); i++) {
    muster_file >> temp;

    #ifdef DEBUG
    cout << "\n";
    cout << i;
    cout << ": "<< temp << " ";
    #endif

    if (muster_file.eof()) {
      break;
    }

    pname[i] = new char [strlen(temp)+1];
    (void) strcpy(pname[i], temp);
    ipattern[i] = new float[ninputs];
    // Einlesen der Muster fuer Test und Lernen ************************
    for (j = 0; j < ninputs; j++) {
      muster_file >> temp;
      if (muster_file.eof()) {
        break;
    }
      #ifdef DEBUG
      cout << temp;
      #endif
      if (muster_file.eof()) {
        rval = put_error("Struktur der Muster-Datei entspricht nicht der Definition! (eof)");
          npatterns = i;

        return(rval);
      }
      if (strcmp(temp,"+") == 0) ipattern[i][j] = (float) 1.0;
      else if (strcmp(temp,"-") == 0) ipattern[i][j] = (float) -1.0;
      else if (strcmp(temp,".") == 0) ipattern[i][j] = (float) 0.0;
      else if (sscanf(temp,"%f",&ipattern[i][j]) != 1) {
        rval = put_error("Struktur der Muster-Datei entspricht nicht der Definition!");
          npatterns = i;

        return(rval);
      }
    }
    #ifdef DEBUG
    cout << " ";
    #endif
    // Referenzmuster ******************************************************
    // nur einlesen wenn pairs==1
    if (pairs) {
      tpattern[i] = new float[noutputs];
      for (j = 0; j < noutputs; j++) {
        
        muster_file >> temp;
        #ifdef DEBUG
        cout << temp;
        #endif
        if (muster_file.eof()) {
          rval = put_error("Struktur der Muster-Datei entspricht nicht der Definition! (eof)");
            npatterns = i;

          return(rval);
        }
        if (strcmp(temp,"+") == 0) tpattern[i][j] = (float) 1.0;
        else if (strcmp(temp,".") == 0) tpattern[i][j] = (float) 0.0;
        else if (strcmp(temp,"-") == 0) tpattern[i][j] = (float) -1.0;
        else if (sscanf(temp,"%f",&tpattern[i][j]) != 1) {
          rval = put_error("Struktur der Muster-Datei entspricht nicht der Definition!");
            npatterns = i;

          return(rval);
        }
      }
    }
  }
  rval = CONTINUE;

  npatterns = i;

  return(rval);
}


